mod exec;
mod rpc {
    tonic::include_proto!("prymn");
}

use std::{
    pin::Pin,
    sync::{Arc, Mutex},
};

use sysinfo::{CpuExt, DiskExt, System, SystemExt};
use tokio_stream::{wrappers::ReceiverStream, Stream, StreamExt};
use tonic::{transport::server::Router, Request, Response, Status};

pub struct Server {
    sys: Arc<Mutex<System>>,
}

type Result<T> = std::result::Result<T, tonic::Status>;

#[tonic::async_trait]
impl rpc::agent_server::Agent for Server {
    async fn echo(&self, req: Request<rpc::EchoRequest>) -> Result<Response<rpc::EchoResponse>> {
        Ok(Response::new(rpc::EchoResponse {
            message: req.into_inner().message,
        }))
    }

    async fn get_sys_info(&self, _: Request<()>) -> Result<Response<rpc::SysInfoResponse>> {
        let sys = self.sys.lock().unwrap();

        let cpus = sys
            .cpus()
            .iter()
            .map(|cpu| rpc::sys_info_response::Cpu {
                freq_mhz: cpu.frequency(),
                usage: cpu.cpu_usage(),
            })
            .collect();

        let disks = sys
            .disks()
            .iter()
            .map(|disk| rpc::sys_info_response::Disk {
                name: disk.name().to_string_lossy().into_owned(),
                total_bytes: disk.total_space(),
                avail_bytes: disk.available_space(),
                mount_point: disk.mount_point().to_string_lossy().into_owned(),
            })
            .collect();

        let response = Response::new(rpc::SysInfoResponse {
            uptime: sys.uptime(),
            hostname: sys.host_name().unwrap_or_default(),
            os: sys.long_os_version().unwrap_or_default(),
            mem_total_bytes: sys.total_memory(),
            mem_avail_bytes: sys.available_memory(),
            swap_total_bytes: sys.total_swap(),
            swap_free_bytes: sys.free_swap(),
            cpus,
            disks,
        });

        Ok(response)
    }

    type ExecStream = Pin<Box<dyn Stream<Item = Result<rpc::ExecResponse>> + Send + Sync>>;

    async fn exec(&self, req: Request<rpc::ExecRequest>) -> Result<Response<Self::ExecStream>> {
        let rpc::ExecRequest { program, args } = req.into_inner();

        match exec::exec(&program, &args) {
            Ok(receiver) => {
                let stream = ReceiverStream::new(receiver).map(|inner| {
                    Ok(rpc::ExecResponse {
                        stdout: inner.stdout.unwrap_or_default(),
                        stderr: inner.stderr.unwrap_or_default(),
                    })
                });

                Ok(Response::new(Box::pin(stream)))
            }
            Err(err) => Err(Status::failed_precondition(err.to_string())),
        }
    }
}

pub fn new_server() -> Router {
    let server = Server {
        sys: Arc::new(Mutex::new(System::new_all())),
    };

    tonic::transport::Server::builder().add_service(rpc::agent_server::AgentServer::new(server))
}
